/*-
 * Shared Object Definition.
 */

#ifndef ShObjDef_h
#define ShObjDef_h


typedef
struct namelist {
	char    **names;
	int       count;
} NameList;

#define NNames(nm)  (nm)->count
#define NameN(nm,n) (nm)->names[(n)]

/*-
 * Symbols generated in host and target libraries
 */
typedef
struct TargetSymbols {
	char           *import;  /* target import section */
	char           *bind;    /* target bind section */
	char           *exp0;    /* target export begin */
	char           *exp1;    /* target export end   */
} TargetSyms;
typedef
struct HostSymbols {
	char           *import; /* host import section */
	char           *bind;   /* host bind section */
	char           *export; /* host export section */
} HostSyms;

typedef
struct ShObjDef {
	char           *target;      /* name of target library */
	char           *host;        /* name of host library */
	char           *prefix;      /* Prefix for generated symbols */
	TargetSyms      tar_names;
	HostSyms        host_names;
	NameList        branch;      /* branch list */
	NameList        objs;        /* object files bound directly */
	NameList        obj_nol;     /* object files (libs) not bound */
	NameList        init[2];     /* init list */
	NameList        hide;	 /* hide list */
	NameList        export;      /* export list */
	ulong_t         jmptab_offs; /* offset of jump table (4k) */
	ulong_t         text_addr;   /* start of code */
	long            text_len;    /* length of code */ 
	ulong_t         data_addr;   /* start of data */
	long            data_len;    /* length of data */
	long            image_size;  /* total memory to reserve for image */
} ShObjDef;

extern int        init_nlist(NameList *nm);
extern int        put_nlist(NameList *nm, char *p);

extern int        ShObjDef_Init(ShObjDef *so);
extern int        ShObjDef_Load(ShObjDef *so, char *pname, FILE *fp);

#endif
