
#include <libc.h>
#include <process.h>
#include <sys/wait.h>
#include "mkshlib.h"

static char    *cc_cmd = "/bin/cc -Q -3 -g1";
static char    *ld_cmd = "/bin/cc -Q -3 -mf -g";

int
cc(char *src, char *dest, char *opts)
{
	char           *buf;
	char           *extraopt = " -c -I../../hdr -Wc,-s ";
	int             t;

	diag_msg(DIAG_DEBUGLEVEL, "compiling '%s' into '%s'\n", src, dest);

	t = strlen(src) + strlen(dest) + strlen(opts) + strlen(cc_cmd)
	     + strlen(extraopt) + 32;

	if ((buf=malloc(t)) == 0) {
		fatalerr("(cc) -- no memory for compile options (%d)\n",t);
	}

	sprintf(buf, "%s %s %s -fo=%s %s",
		cc_cmd, extraopt, opts ? opts : "", dest, src);
	if ((t = runcmd(buf, 0)) > 0) {
		warning("errors/warnings occurred compiling '%s'", src);
	}
	free(buf);
	return t != 0;
}

int
archive(char *arch, char *sfile, char *opts)
{
	char           *buf;
	int             t;
	t = strlen(arch) + strlen(sfile) + strlen(opts) + 32;
	if ((buf=malloc(t)) == 0) {
		fatalerr("(archive) -- no memory for command buffer(%d)\n", t);
	}
	sprintf(buf, "/bin/wlib -q -n -b -c %s %s %s",
		opts ? opts : "", arch, sfile);
	if ((t = runcmd(buf, 1)) > 0) {
		warning("errors/warnings occured building archive '%s'", arch);
	}
	free(buf);
	return t;
}

int
ld(char *src, char *dest, char *opt, ulong_t base_offset)
{
	char            *buf;
	int             t;

	t = strlen(ld_cmd) + strlen(src) + strlen(dest) + strlen(opt) + 128;
	if ((buf=malloc(t)) == 0) {
		fatalerr("(ld) -- no memory for command buffer (%d)\n", t);
	}
	diag_msg(DIAG_DEBUGLEVEL, "linking %s into %s with %s\n", src, dest, opt);
	sprintf(buf, "%s -@0x%x -o %s %s %s", ld_cmd, base_offset, dest, src, opt);
	diag_msg(DIAG_DEBUGLEVEL, "%s\n", buf);
	if ((t = runcmd(buf, 0)) > 0) {
		warning("warnings/errors occurred linking '%s'", dest);
	}
	free(buf);
	return t;
}

int
runcmd(char *buf, int expecterr)
{
	pid_t           chldpid;
	int             retstat;
	int             rwait;
	int             n;
	int             tsize;
	int             std_out[2];
	char            rbuf[1024];

	if (pipe(std_out) == -1) {
		fatalerr("pipe failed (%s)", strerror(errno));
	}
	if ((chldpid = fork()) == -1) {
		fatalerr("fork failed (%s) trying to run '%s'",
			 strerror(errno), buf);
	} else if (chldpid == 0) {
		close(std_out[0]);
		dup2(std_out[1], 1);
		dup2(std_out[1], 2);
		close(std_out[1]);
		execl("/bin/sh", "/bin/sh", "-c", buf, 0);
		fatalerr("exec failed (%s) trying to run '%s'",
			 strerror(errno), buf);
	}
	close(std_out[1]);
	tsize = 0;
	while ((n = read(std_out[0], rbuf, sizeof rbuf)) != 0) {
		if (n == -1)
			continue;
		if (tsize == 0) {
			diag_msg(DIAG_INFOLEVEL, "(%s) - output begin\n",
				buf);
		}
		diag_write(DIAG_INFOLEVEL, rbuf, n);
		if (!program_flags.quiet) {
			write(2, rbuf, n);
		}
		tsize += n;
	}
	close(std_out[0]);
	if (tsize) {
		diag_msg(DIAG_INFOLEVEL, "output end\n");
	}
	while ((rwait = waitpid(chldpid, &retstat, 0)) != chldpid && rwait != -1);
	if (rwait == -1) {
		fatalerr("waitpid failed (%s) running command '%s'",
			 strerror(errno), buf);
	} else if (WIFEXITED(retstat)) {
		if (WEXITSTATUS(retstat) != 0 && expecterr == 0)
			fatalerr("'%s' exitted(%d)", buf, WEXITSTATUS(retstat));
	} else {
		fatalerr("'%s' abnormal exit(%d)", buf, WTERMSIG(retstat));
	}
	return tsize;
}
