/*-
Module:   Host
Purpose:  Produce the host library
Input:    ShObjDef
Output:   Host Library.

Processing:
 The host library is comprised of the following:

 1.  An error handling module.
 2.  A Linker-Control module.
 3.  An AUTO-INIT module.
 4.  A set of ABSOLUTE modules.

The ABSOLUTE modules reference the AUTO-INIT one to bring it
in before main().
An ABSOLUTE module is either a jump table entry or an exported
data item.

OMF files are generated directly -- There currently isn't a
stable assembler in QNX.

*/

#include <libc.h>
#include "mkshlib.h"
#include "codegen.h"

static
BldErrFunc(FILE * f)
{
	char           *src, *dest;
	FILE           *fp;
	char           *macp = getmac("HostErrorHandler");

	if (macp == 0) {
		warning("host_err: No Macro for Error Handler\n");
		return 1;
	}
	if (!(src = temp(TEMP_HOST "_err", ".c")) != 0) {
		fatalerr("host_err: cannot create temp file");
	}
	if (!(dest = temp(TEMP_HOST "_err", ".o")) != 0) {
		untemp(src, program_flags.no_unlink);
		fatalerr("host_err: cannot create temp file");
	}
	if ((fp = xfopen(src, "w", "host_errfun")) == 0) {
		fatalerr("host_err: cannot create temp file");
	}
	fmacrov(fp, macp, 0);
	fclose(fp);
	cc(src, dest, "");
	untemp(src, program_flags.no_unlink);
	if ((fp = xfopen(dest, "r", "host_export")) == 0) {
		untemp(dest, program_flags.no_unlink);
		return -1;
	}
	fcat(f, fp);
	fclose(fp);
	untemp(dest, program_flags.no_unlink);
	return 0;
}


static
BldInit(FILE * f, ShObjDef *sh)
{
	FILE           *fp;
	char           *src, *dest;
	int             i;
	char           *expmac = getmac("HostExportTable");

	if (expmac == 0) {
		fatalerr("HostExport: No macro for export table");
		return -1;
	}
	if (!(src = temp(TEMP_HOST, ".c"))) {
		fatalerr("host_export, cannot create temporary file");
	}
	diag_msg(DIAG_DEBUGLEVEL, "%s is for building the host init list\n", src);
	if (!(dest = temp(TEMP_HOST, ".o"))) {
		untemp(src, program_flags.no_unlink);
		fatalerr("host_export, cannot create temporary file");
	}
	diag_msg(DIAG_DEBUGLEVEL, "%s is the object file for %s\n", dest, src);

	if ((fp = xfopen(src, "w", "host_export")) == 0) {
		fatalerr("host_export, cannot open temporary file");
	}
	fprintf(fp, "/* warning: machine generated file, do not edit */\n");
	for (i=0; i < sh->init[1].count; i++) {
		fprintf(fp, "extern %s\n;",
			sym_cname(sh->init[1].names[i]));
	}
	fputs("static void *export_tab[] = {\n", fp);
	if (sh->init[1].count) {
		for (i=0; i < sh->init[1].count; i++) {
			fprintf(fp, "(int) &%s%s\n",
				//sym_cname(sh->init[1].names[i]),
				sh->init[1].names[i],
				i == sh->init[1].count-1 ? "" : ",");
		}
	} else {
		putc('0', fp);
	}
	fputs("};\n", fp);
	fmacrov(fp, expmac,
		sh->prefix,
		"export_tab",
		sh->host_names.bind,
		0);
	fclose(fp);
	cc(src, dest, "");
	untemp(src, program_flags.no_unlink);
	if ((fp = xfopen(dest, "r", "host_export")) == 0) {
		untemp(dest, program_flags.no_unlink);
		return -1;
	} else {
		fcat(f, fp);
		fclose(fp);
		untemp(dest, program_flags.no_unlink);
		return 0;
	}
}
static char      *import_symbol;
static ulong_t    jmptable_base;
static int
HostSymbol(char *name, Symbol *p, FILE *f)
{
	ulong_t     addr;
	if (IS_BRANCH(p)) {
		addr = jmptable_base + p->branch_index*8;
	} else if (p->is_exported) {
		addr = p->address;
	} else {
		return 0;
	}
	mkabs(f, sym_ename(name), import_symbol, addr);
	return 0;
}


mkhost(ShObjDef *sh)
{
	char           *s;
	FILE           *f;
	char           *name;
	char           *xibmod;
	char           *xibsym;

	if (!(s = temp(TEMP_HOST, ".o"))) {
		fatalerr("mkhost:cannot get temp file");
	}
	diag_msg(DIAG_DEBUGLEVEL, "%s is the container for objects in the host library\n", s);
	if ((f = xfopen(s, "w", "mkhost")) == 0) {
		fatalerr("mkhost: cannot open temp file");
	}
	mkdummy(f, sh->text_addr, sh->image_size);

	name = malloc(PATH_MAX+1);
	xibmod = malloc(PATH_MAX+1);
	xibsym = malloc(PATH_MAX+1);

	strcpy(name, sh->host_names.bind);   strcat(name, "_xibsym");
	strcpy(xibmod, "__(initcode)__");    strcat(xibmod, sh->host_names.bind);
	strcpy(xibsym, sh->host_names.bind); strcat(xibsym, "_");

	mkxib(f, xibmod, xibsym, name, -1, !program_flags.nofloat);

	free(xibmod);
	free(xibsym);

	import_symbol = name;
	jmptable_base = sh->text_addr + sh->jmptab_offs;
	sym_apply(HostSymbol, f);

	import_symbol = 0;
	free(name);

	BldInit(f, sh);
	BldErrFunc(f);

	fclose(f);
	/* -- */
#undef unlink
	unlink(sh->host);
	archive(sh->host, s, "");
	untemp(s,program_flags.no_unlink);
	return 0;
}
