
/*-
 * omf.h:   Structures and definitions for manipulating
 * Object Module Format files.
 *
 * The OMF is a record based format, the functions beginning
 * with "Omf" either read records or specific types within
 * records.
 * Function beginning with "omf" either write speicific types
 * to a record or an entire record.
 */

#ifndef omf_h
#define omf_h

#ifndef PCOBJ_H
#include "pcobj.h"
#endif

#ifndef __TYPES_H_INCLUDED
#include <sys/types.h>
#endif


#ifndef _STDIO_H_INCLUDED
#include <stdio.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


typedef
struct objrec {
	uchar_t    rtype;
	unsigned   rlen;
	uchar_t   *rbufp;
	int        buflen;
	int        curp;
	int        ms386:1;

} OmfRec;

#define OmfEndRec(o)    ((o)->curp >= (o)->rlen-1)
#define OmfGetByte(o)   ((o)->rbufp[(o)->curp++])
#define OmfUnGet(o)     ((o)->curp--)
typedef unsigned short   u16;
typedef unsigned long    u32;


extern OmfRec  *OmfNewRec(void);
extern int      OmfFreeRec(OmfRec *o);

extern int      OmfRead(FILE *fp, OmfRec *ob);

extern int      OmfGetName(OmfRec *o, char *bufp);

/*-
 * Basic data types
 */
extern u16      OmfGetIndex(OmfRec *o);
extern u32      OmfGetDWord(OmfRec *o);
extern u16      OmfGetWord(OmfRec *o);
extern u32      OmfGetEither(OmfRec *o);


/*-
 * These write records.
 */


typedef struct {
	uchar_t	*bufp;
	int	len;
	int	curp;
} orec_t;

#define	omf_clear(r)	((r)->curp = 0)
#define omf_len(r)	((r)->curp)
#define omf_prevb(r)	((r)->bufp[(r)->curp-1])
#define omf_type(r)     ((r)->bufp[0])

/* these manipulate fields of a record */
extern int omf_initorec(orec_t *r);
extern orec_t *omf_allocrec(void);
extern int omf_growrec(orec_t *r , int n);
extern int omf_putbyte(orec_t *r , int val);
extern int omf_putword(orec_t *r , int u);
extern int omf_putdword(orec_t *r , long u);
extern int omf_puteither(orec_t *r , long u);
extern int omf_putname(orec_t *r , char *name , int len);
extern int omf_putindex(orec_t *r , unsigned index);

/* these put data into a file */
extern int omf_putc(int c, FILE *fp, int *csum);
extern int omf_putw(int w, FILE *fp, int *csum);
extern int omf_putl(long w, FILE *fp, int *csum);
extern int omf_putrec(FILE *fp, orec_t *r,int type);

/* these generate entire records into a file */
extern int omf_modend(FILE *fp, int ismain);
extern int omf_msobj(FILE *fp);
extern int omf_lnames(FILE *fp, char **names, int nnames);
extern int omf_freerec(orec_t *r);
extern int omf_theader(FILE *fp, char *name);


#ifdef __cplusplus
};
#endif

#endif
