/*
 * main: parse options, setup global variables, invoke mkshlib
 */
#include <libc.h>
#include "mkshlib.h"

static char ident[] = "$Id: main.c,v 1.7 1996/11/25 16:32:58 steve Exp $";

char             *program_name = "mkshlib";
struct runflags   program_flags;

/*
	names of the target, specfile and host libraries
*/
static char             *target;
static char             *specfile;
static char             *host;
/*-
	directories for libraries and temporary files
*/
char           *libpath = "/usr/lib";


/*- options for loader compiler */
char            loader_opts[LINE_MAX] = "-b -g ";
char            ccomp_opts[1024] = "-g1 -Wc,-s";

/*- default type is data */
char           *ass_type = "";

long            base_address = 0x1000;	/* jump table offset */




static char     *bname = 0;
static char     *skelnam = DEFAULT_SKELETON;


get_options(int argc, char **argv)
{
	int            c;

	program_flags.no_unlink  = 0;
	program_flags.print_stats = 0;
	program_flags.use_gas     = 0;
	program_flags.quiet       = 0;
	program_flags.set_init   = 1;
	program_flags.nofloat     = 0;
	while ((c = getopt(argc, argv, "qns:t:h:L:**fvTCDGUd:W:B:S:")) != -1) {
		switch (c) {

		case 'q': /* make no noise during generation */
			program_flags.quiet = 1;
			break;

		case 'n': /* do not generate a new target library */
			target = 0;
			break;

		case 's': /* specification file (or stdin) */
			specfile = optarg;
			break;

		case 't': /* target library name (client) */
			target = optarg;
			break;

		case 'h': /* host library name */
			host = optarg;
			break;

		case 'L': /* library path */
			strcat(loader_opts, " -L");
			strcat(loader_opts, optarg);
			strcat(loader_opts, " ");
			break;

		case 'f':
			program_flags.nofloat = 1;
			break;

		case 'D': /* default symbol type is data */
			ass_type = "";
			break;

		case 'C': /* default symbol type is function */
			ass_type = "()";
			break;

		case 'T': /* Disable init table auto-load */
			program_flags.set_init = 0;
			break;

			/* pass option to compilation passes */
		case 'W':
			switch (*optarg) {
			case 'c':
			case 'C':
			case 's':
			case 'S':
				warning("sorry no support for '%c' options",
					*optarg);
				break;
			case 'l':
			case 'L':
				if (optarg[1] == ',') {
					strcat(loader_opts, " -Wl,\"");
					strcat(loader_opts, optarg + 2);
					strcat(loader_opts, "\" ");
				} else {
					warning("garbled loader option '%s' ignored",
						optarg);
				}
				break;
			default:
				warning("unknown processing specifier '%c' ignored",
					*optarg);
			}
			break;

		case 'G':
			program_flags.use_gas = 1;
			break;

		case 'B': /* generate a trace log */
			bname = optarg;
			break;

		case 'S':
			skelnam = optarg;
			break;

		case 'd':
			diag_file(optarg);
			diag_level(DIAG_INFOLEVEL);
			break;

		case 'v': /* verbose -- output statistics of library build */
			program_flags.print_stats = 1;
			diag_level(DIAG_DEBUGLEVEL);
			break;

		case 'U':
			program_flags.no_unlink = 1;
			break;

		default:	/* getopt makes noise for us */
			return -1;
		}
	}
	return 0;
}

/*-
 * Fix_name returns a version of "nam" which is a valid C identifier
 */
char *
fix_name(char *nam)
{
	char      *s, *t;
	for (s=nam; *s && !(isalpha(*s) || *s == '_'); s++)
		;
	for (t=s; *t && (isalnum(*t) || *t == '_'); t++)
		;
	*t = '\0';
	return s;
}


init_names(ShObjDef *sh)
{
	char            buf[PATH_MAX];
	char           *nam = sh->prefix;

	sprintf(buf, "_tar%s_import", nam);
	sh->tar_names.import = strdup(buf);

	sprintf(buf, "_tar%s_bind", nam);
	sh->tar_names.bind = strdup(buf);

	sprintf(buf, "_tar%s_exportbeg", nam);
	sh->tar_names.exp0 = strdup(buf);

	sprintf(buf, "_tar%s_exportend", nam);
	sh->tar_names.exp1 = strdup(buf);

	sprintf(buf, "_host%s_import", nam);
	sh->host_names.import = strdup(buf);

	sprintf(buf, "_cl%s_bind", nam);
	sh->host_names.bind = strdup(buf);

	sprintf(buf, "_cl%s_export", nam);
	sh->host_names.export = strdup(buf);

	if (!sh->host) {
		sprintf(buf, "%s_s3r.lib", nam);
		sh->host = strdup(buf);
	}
	if (!sh->target) {
		sprintf(buf, "%s_s", nam);
		sh->target = strdup(buf);
	}
	return 0;
}

static
add_libs(NameList *nol)
{
	int	i;
	for (i=0; i < nol->count; i++) {
		strcat(loader_opts, nol->names[i]);
		strcat(loader_opts, " ");
	}
}

main(int argc, char **argv)
{
	FILE            *fp;
	ShObjDef         objdef;

	ShObjDef_Init(&objdef);

	if (get_options(argc, argv) < 0) {
		return 1;
	}

	if (loadskel(skelnam) == -1) {
		fatalerr("cannot load skeleton file '%s' (%s)",
			 skelnam, strerror(errno));
	}
	if (!specfile) {
		fatalerr("must provide specification file");
	} else if (strcmp(specfile, "-") == 0) {
		fp = stdin;
		specfile = "stdin";
	} else if ((fp=fopen(specfile, "r")) == 0) {
		fatalerr("cannot open specification file '%s' (%s)",
			specfile, strerror(errno));
	}

	if (target) {
		bname = strdup(basename(target));
		objdef.target = strdup(target);
	} else {
		bname = strdup(basename(specfile));
	}
	if (host) {
		objdef.host = strdup(host);
	}
	objdef.prefix = fix_name(strdup(bname));

	if (ShObjDef_Load(&objdef, specfile, fp) == -1) {
		fatalerr("cannot load specification definition");
	}
	add_libs(&objdef.obj_nol);

	fclose(fp);

	init_names(&objdef);

	mktarget(&objdef);
	mkhost(&objdef);

	return 0;
}
