
#include <string.h>
#include <stdlib.h>
#include "coff.h"

static char *
storclass(int sclass)
{

static struct {
	int	key;
	char	*string;
} sclass_tab[] = {
	{	C_EFCN, "End of Function" },
	{	C_NULL, "" },
	{	C_AUTO, "auto" },
	{	C_EXT, "extern" },
	{	C_STAT, "static" },
	{	C_REG, "register" },
	{	C_EXTDEF, "public" },
	{	C_LABEL, "label" },
	{	C_ULABEL, "undefined label" },
	{	C_MOS, "structure member" },
	{	C_ARG, "function argument" },
	{	C_STRTAG, "structure tag" },
	{	C_MOU, "union member" },
	{	C_TPDEF, "type definition" },
	{	C_USTATIC, "undefined static" },
	{	C_ENTAG, "enum tag" },
	{	C_MOE, "enum member" },
	{	C_REGPARM, "register parm" },
	{	C_FIELD, "bit field" },
	{	C_BLOCK, "block delim" },
	{	C_FCN, "func delim" },
	{	C_EOS, "end structure" },
	{	C_FILE, "file name" },
	{	C_LINE, "line" },
	{	C_ALIAS, "duplicate tag" },
	{	C_HIDDEN, "hidden" },
	{	C_SHADOW, "shadow" }
};
	for (int i=0; i < sizeof sclass_tab/sizeof(sclass_tab[0]); i++) {
		if (sclass_tab[i].key == sclass)
			return sclass_tab[i].string;
	}
	return "<unknown>";
}

static char *
type(int n_type, char *nam)
{
static char buf[240];
	int	rtype = n_type;
	char	prefix[40], postfix[40];
	prefix[0] = postfix[0] = buf[0] = '\0';

	for (int i=0; i < 6; i++) {
		if (ISPTR(rtype)) {
			strcat(prefix,"* ");
		} else if (ISFCN(rtype)) {
			strcat(postfix,"()");
		} else if (ISARY(rtype)) {
			strcat(postfix,"[]");
		} else
			break;
		rtype = DECREF(rtype);
	}
	switch (BTYPE(n_type)) {
	case	T_NULL:		strcat(buf,"anon");	break;
	case	T_ARG:		strcat(buf,"arg");	break;
	case	T_CHAR:		strcat(buf,"char");	break;
	case	T_SHORT:	strcat(buf,"short");	break;
	case	T_INT:		strcat(buf,"int");	break;
	case	T_LONG:		strcat(buf,"long");	break;
	case	T_FLOAT:	strcat(buf,"float");	break;
	case	T_DOUBLE:	strcat(buf,"double");	break;
	case	T_STRUCT:	strcat(buf,"struct");	break;
	case	T_UNION:	strcat(buf,"union");	break;
	case	T_ENUM:		strcat(buf,"enum");	break;
	case	T_MOE:		strcat(buf,"enum element"); break;
	case	T_UCHAR:	strcat(buf,"uchar");	break;
	case	T_USHORT:	strcat(buf,"ushort");	break;
	case	T_UINT:		strcat(buf,"uint");	break;
	case	T_ULONG:	strcat(buf,"ulong");	break;
	}
	strcat(buf," "); strcat(buf,prefix);
	strcat(buf," "); strcat(buf,nam);
	strcat(buf,postfix);
	return buf;
}

Symbol::print_sym(FILE *f, int symno)
{
	if (symno >= nused)
		return -1;
	symp	*p = stab+symno;
	fprintf(f,"%d: %s sclass=%d, type=%d, section=%d, value=%d, aux=%d\n",
		symno, p->sym.n_nptr, p->sym.n_sclass, p->sym.n_type,
		p->sym.n_scnum, p->sym.n_value, p->sym.n_numaux);
	return 0;
}

int
Symbol::print(FILE *f)
{
	fprintf(f,"%d symbols:\n",nused);
	for (int i=0; i < nused; i++) {
		print_sym(f, i);
		i += stab[i].sym.n_numaux;
	}
	return 0;
}
