#include "coff.h"

coff::coff(char *fname)
{
	clear();
	if ((fp=fopen(fname,"r")) == NULL) {
		state = -1;
		return;
	}
	load();
}

coff::coff(FILE *f)
{
	clear();
	if ((fp = f) != 0) {
		load();
	} else {
		state= -1;
	}
}

char *
coff::loadstrs()
{

	if (header.f_flags & F_LSYMS) {
		return 0;
	}
	long soffs = header.f_symptr + header.f_nsyms*SYMESZ;
	long slen = filelen - soffs;
	char *strings = new char [slen+1];
	if (strings == 0) {
		error = errno;
		state = -1;
		return 0;
	}
	fseek(fp, soffs, SEEK_SET);
	fread(strings,1,slen,fp);
	return strings;

}

int
coff::loadsyms()
{
	if (!symtab) {
		return 0;
	}
	fseek(fp, header.f_symptr, SEEK_SET);
	for (int i=0; i < header.f_nsyms; i++) {
		SYMENT  syment;
		fread(&syment, SYMESZ, 1, fp);
		symtab->install(&syment,0);
	}
	return 0;
}

int
coff::loadsects()
{
	long offset = sizeof header + header.f_opthdr;
	fseek(fp, offset, SEEK_SET);
	sections = new Sect[header.f_nscns];
	for (int i=0; i < header.f_nscns; i++) {
		sections[i].load(fp,offset); 
		offset += sizeof(SCNHDR);
	}
	return 0;
}


/*
 * load the headers from a coff file.
 */
int coff::load(void)
{
	state = 0;
	fseek(fp,0,SEEK_END);
	filelen = ftell(fp);
	fseek(fp,0,SEEK_SET);
	if (fread(&header, sizeof header, 1, fp) != 1) {
		error = errno;
		return state = -1;
	}
	if (!ISCOFF(header.f_magic)) {
		error = ENOEXEC;
		return state = -1;
	}
	if (header.f_opthdr) {
		if (fread(&aout, sizeof aout, 1, fp) != 1) {
			error = errno;
			return state = -1;
		}
	} else {
		memset(&aout, 0, sizeof aout);
	}
	loadsects();
	/*
	 * load string and symbol table
	 */

	char *str = loadstrs();
	if (str) {
		symtab = new Symbol(str);
	}
	loadsyms();
	return 0;
}
