/*-
 * ldf_nlist: extract name list from lmf file.
 */

#include <libc.h>
#include <sys/lmf.h>
#include "ldfile.h"

#if 0
static void
print_symlist(LdfName *p)
{
	while (p) {
		if (p->type & _SY_STATIC) { printf("static "); }
		printf((p->type & _SY_CODE) ? "code " : "data ");
		printf("%s segment %d offset %lx\n", p->name, p->segno, p->offset);
		p=p->next;
	}
}
#endif

static LdfName *
addsym(LdfName *root, char *name, int segno, unsigned long offs, int type)
{

	LdfName *r;

	r = malloc(sizeof *r);
	r->name = strdup(name);
	r->segno = segno >> 3;	/* map out privity, ti */
	r->offset = offs;
	r->type = type;
	r->next = root;
	return r;
}

static LdfName *
ldf_readnlist(FILE * fp)
{
	struct {
		unsigned short  signature;
		unsigned short  ldf_version, obj_version;
		unsigned short  lang_size, segment_size;
		unsigned long   debug_size;
	}               m;	/* master debug header */
	long            section;
	struct {
		unsigned long   mod_offset, gbl_offset, addr_offset;
		unsigned long   section_size;
		unsigned short  section_id;
	}               s;	/* section debug header */
	struct {
		unsigned long   addr_off;
		unsigned short  addr_seg;
		unsigned short  mod_offset;
		unsigned char   s_type;
		unsigned char   s_len;
	}               g;	/* global symbol header */

	LdfName *root = 0;

	fseek(fp, -(long) sizeof m, SEEK_END);
	fread(&m, sizeof m, 1, fp);

	if (m.signature != 0x8386)
		return 0;

	if (m.ldf_version > 3)
		return 0;

	fseek(fp, m.lang_size + m.segment_size - m.debug_size, SEEK_END);
	section = ftell(fp);
	fread(&s, sizeof s, 1, fp);

	fseek(fp, section + s.gbl_offset, SEEK_SET);
	do {
		char		name[1024];

		fread(&g, sizeof g, 1, fp);
		fread(name, g.s_len, 1, fp);
		name[g.s_len] = 0;
		root = addsym(root, name, g.addr_seg, g.addr_off, g.s_type);

	} while (ftell(fp) < section + s.addr_offset);
	return root;
}

LdfName *
ldf_nlist(char *fname)
{
	LdfName *s;
	LdfName *sp;
	FILE *fp;
	LdInfo  ldi;

	if (ldf_seglen(fname, &ldi) == -1) {
		return 0;
	}
	if ((fp=fopen(fname, "r")) == 0) {
		return 0;
	}
	if ((s=ldf_readnlist(fp)) == 0) {
		fclose(fp);
		return 0;
	}
	fclose(fp);

	/*-
	 * Fix the addresses to the actual (load) addresses.
	 */
	for (sp=s; sp; sp = sp->next) {
		if (sp->segno) {
			sp->offset += (ldi.ld_tsize+4095) & ~4095;
		}
		sp->offset += ldi.ld_vaddr;
	}
	return s;
}

int
ldf_del_nlist(LdfName *nl)
{
	while (nl) {
		LdfName *sp = nl->next;
		free(nl->name);
		free(nl);
		nl = sp;
	}
	return 0;
}

int
ldf_walk_nlist(LdfName *nl, int (*f)(LdfName *x, void *p), void *p)
{
	while (nl) {
		f(nl, p);
		nl = nl->next;
	}
	return 0;
}
