#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "map.h"

maptab         *
map_new(int n)
{
	maptab         *mp;
	if ((mp = malloc(sizeof *mp)) == 0) {
		return 0;
	}
	mp->nentries = n;
	if ((mp->maplist = calloc(sizeof *mp->maplist, mp->nentries)) == 0) {
		free(mp);
		return 0;
	}
	return mp;
}

int
map_hash(char *name, int n)
{
	int             i;
	int             tot;
	for (tot = 0, i = 0; name[i]; tot += name[i++]);
	return tot % n;
}

void           *
map_lookup(maptab * mp, char *name)
{
	int             n = map_hash(name, mp->nentries);
	map_t          *m = mp->maplist + n;
	while (m && m->key && strcmp(m->key, name) != 0)
		m = m->next;
	return m ? m->data : 0;
}

void           *
map_install(maptab * mp, char *name, void *data)
{
	int             n = map_hash(name, mp->nentries);
	map_t          *m = mp->maplist + n;

	if (!m->key) {
		m->key = name;
		return m->data = data;
	}
	while (m->next && strcmp(m->key, name) != 0)
		m = m->next;
	if (strcmp(m->key, name) == 0) {
		return m->data = data;
	}
	m->next = malloc(sizeof *m);
	m = m->next;
	m->key = name;
	m->next = 0;
	return m->data = data;
}

int
map_apply(maptab * mp, int (*f) (), void *p)
{
	int             n;
	map_t          *m;
	for (n = 0; n < mp->nentries; n++) {
		for (m = mp->maplist + n; m && m->key; m = m->next) {
			(*f) (m->key, m->data, p);
		}
	}
	return 0;
}
