/*-
 * mkabs:   This module produces a single omf-module which declares a
 *    single absolute symbol.
 *    There is an 'optional' parameter; which is the name of a symbol
 *    which should be externally 'referenced' by this module.   Note
 *    that this reference is a "dummy" which forces the linker to 
 *    bring said module into the load file.
 */

#include <libc.h> 
#include "codegen.h"
#include "omf.h"

static char *segnames[] = {
	"",
	"CODE",
	"DATA",
	"BSS",
	"DGROUP", 
	"_TEXT", 
	};
	
#define _SEGLEN 6


mkabs(FILE *fp, char *name, char *xib, long addr)
{
	orec_t             *r = omf_allocrec();
	char                hdrname[256];

	strcpy(hdrname, "__(absolute)__");
	strcat(hdrname, name);

	omf_theader(fp,hdrname);
	omf_msobj(fp);
	omf_lnames(fp,segnames, 6);
/* define code segment */
	omf_putbyte(r, (ALIGN_BYTE<<5)|(COMB_ADDOFF<<2)|1);
	omf_puteither(r, _SEGLEN);	/* length of code  segment */
	omf_putbyte(r, 6);	/* name of segment */
	omf_putbyte(r, 2);	/* segment class-name (lnames index) */
	omf_putbyte(r, 1);	/* overlay flags */
	omf_putrec(fp,r,CMD_SEGD32);
	omf_clear(r);

	if (xib) {
		omf_putname(r, xib, strlen(xib));
		omf_putindex(r, 0);
		omf_putrec(fp,r,CMD_EXTDEF);
		omf_clear(r);
	}

/* define absolute symbol */
	omf_putindex(r, 0);	/* group	*/
	omf_putindex(r, 0);	/* segment	*/
	omf_putword(r, 0);	/* frame	*/
	omf_putname(r, name, strlen(name));
	omf_puteither(r, addr);
	omf_putindex(r, 0);
	omf_putrec(fp, r, CMD_PUBD32);
	omf_clear(r);

	omf_freerec(r);
	
/* put module end def */	
	omf_modend(fp, 0);
	return 0;
}
