

#include <libc.h>
#include "codegen.h"
#include "omf.h"

/* Indices are base 1 in ms-omf */

static char *segnames[] = {
	"",
	"CODE",
	"DATA",
	"BSS",
	"BEGTEXT",
	"DGROUP",
	"_TEXT"
};

enum {
	CodeSegno = 2,
	DataSegno = 3,
	BSSSegno  = 4,
	BegCodeSegno = 5,
	DataGroup = 6,
	TextGroup = 7,
};

static int	name_munge=1;
// static const unsigned char jmp_opcode[] = { 0xe9, 0x00, 0x00, 0x00, 0x00 };

/*-
 * generate a Relative Fixup (call, jump, ...) for SYMNO at OFFSET.
 * Note: SYMNO is base 1.
 */
#define DIRECT_FIXUP (FIXUPP_FIXUP|(LOC_MS_OFFSET_32<<2)|FIXDAT_MBIT)
#define PCREL_FIXUP  (FIXUPP_FIXUP|(LOC_MS_OFFSET_32<<2))

static int
omf_relfixup(orec_t *r, int offset, int symno)
{
	int	fixcode;
	int	frame = 0;

	fixcode = PCREL_FIXUP | ((offset>>8) & 03);
	frame = (F_SEG<<4) | T_EXTWD | 0x04;
//	frame = (F_TARG<<4) | T_EXTWD | 0x04;

	omf_putbyte(r, fixcode);
	omf_putbyte(r, offset & 0xff);
	omf_putbyte(r, frame);
	omf_putindex(r, 1);	/* code seg?!? */
	omf_putindex(r, symno);
	return 0;
}


static int
omf_fixup(orec_t *r, int offset, int symno)
{
	int	fixcode;
	int	frame;

	fixcode = DIRECT_FIXUP | ((offset>>8)&03);
	frame   = (F_TARG<<4) | T_EXTWD | 0x4;

	omf_putbyte(r, fixcode);
	omf_putbyte(r, offset & 0xff);
	omf_putbyte(r, frame);
	omf_putindex(r, symno);
	return 0;
}

static int
omf_ledata_init(orec_t *o, int section, int offset)
{
	section = section;		// not used.
	omf_putindex(o, 1);
	omf_puteither(o, offset);
	return omf_len(o);
}

static int
omf_ledata_done(orec_t *o, FILE *fp)
{
	omf_putrec(fp, o, CMD_LEDA32);
	omf_clear(o);
	return 0;
}

#define OMFRMAX 130
/*-
 * Unobvious cohesion: The symbol base points at the symbol for the
 *                 slib_bind structure.  The first jumptable entry
 *                 points directly at this structure.
 */
static
jump_table(FILE *fp, struct jmptab *jp, int symbase)
{
	orec_t            *code;
	orec_t            *fix;
	int                offset;
	int                i;
	long               coffs = jp->j_offset;

	code = omf_allocrec();
	fix = omf_allocrec();

	omf_ledata_init(code, 1, coffs);

	omf_putdword(code, 0);
	omf_fixup(fix, 0, symbase++);
	omf_putdword(code, 0);
	offset = 2 * sizeof(long);
	coffs += 2 * sizeof(long);
	for (i=0; i < jp->j_nvec; i++) {
		if (omf_len(code) > OMFRMAX) {
			omf_ledata_done(code, fp);

			omf_putrec(fp, fix, CMD_FIXU32);
			omf_clear(fix);

			offset = 0;
			omf_ledata_init(code, 1, coffs);
		}

		omf_putbyte(code, 0xe9);
	/* relative fixup at address n+1, symbol 1 */
		omf_relfixup(fix, offset + 1, i + symbase);
		omf_putdword(code, 0);
	/* pad alignment */
		omf_putbyte(code, 0x90);
		omf_putbyte(code, 0x90);
		omf_putbyte(code, 0x90);

		offset += 8;
		coffs += 8;
	}
	omf_putbyte(code, 0xc3); 	/* return */
	omf_ledata_done(code, fp);

	if (omf_len(fix) > 0) {
		omf_putrec(fp, fix, CMD_FIXU32);
	}
	omf_freerec(fix);
	omf_freerec(code);
	return 0;
}


static int
putnam(orec_t *r, char *nam, int where)
{
	char buf[256];
	if (name_munge) {
		if (where == 0) {
			strcpy(buf, nam);
			strcat(buf, "_");
		} else {
			buf[0] = '_';
			strcpy(buf+1, nam);
		}
		nam = buf;
	}
	omf_putname(r, buf, strlen(buf));
	return 0;
}

mkjumptable(FILE *fp, struct jmptab *jp)
{
	int 	i;
	orec_t	*r = omf_allocrec();
	int     base_sym = 3;


	omf_theader(fp,jp->jl_modnam);
	omf_msobj(fp);
	omf_lnames(fp,segnames, sizeof segnames/sizeof *segnames);


	putnam(r, jp->jl_start, 1);
	omf_putindex(r, 0);
	putnam(r, jp->jl_end, 1);
	omf_putindex(r, 0);
	putnam(r, jp->jl_relocp, 1);
	omf_putindex(r, 0);

	for (i=0; i < jp->j_nvec; i++) {
		if (omf_len(r) > 130) {
			omf_putrec(fp, r, CMD_EXTDEF);
			omf_clear(r);
		}
		putnam(r, jp->j_vec[i], 0);
		omf_putindex(r, 0);
	}
	if (omf_len(r)) {
		omf_putrec(fp, r, CMD_EXTDEF);
		omf_clear(r);
	}
/*-
 * All external names now defined
 */

	/* align combine type */
	omf_putbyte(r, (ALIGN_BYTE<<5)|(COMB_ADDOFF<<2)|1);
	omf_puteither(r, jp->j_offset + 8 * (jp->j_nvec+1) + 1);
	omf_putbyte(r, BegCodeSegno);	/* name of segment */
	omf_putbyte(r, CodeSegno);	/* segment class-name (lnames index) */
	omf_putbyte(r, 1);	/* overlay flags */
	omf_putrec(fp,r,CMD_SEGD32);
	omf_clear(r);

	jump_table(fp, jp, base_sym);


	omf_putbyte(r, 0);
	omf_putbyte(r, 1);
	putnam(r, jp->jl_start, 1);
	omf_puteither(r, jp->j_offset);
	omf_putindex(r,0);

	putnam(r, jp->jl_end, 1);
	omf_puteither(r, jp->j_offset + 8 * (jp->j_nvec+1));
	omf_putindex(r, 0);

	omf_putrec(fp, r, CMD_PUBD32);
	omf_freerec(r);

	omf_modend(fp, 0);
	return 0;
}
