

/*-
genxib: output an "auto-init" module.

 Watcom startup code uses a segment trick in the linker, where the distance
 between two segments in the dgroup (called {xib, xie}) is used to call init
 routines.
 The area of memory contains a structure of the form:
 	unsigned char _reserved;
 	unsigned char priority;
 	unsigned long func-addr;
 In the startup code, it walks this array, calling 'func-addr's' in the order
 of increasing priority field.
 This code generates a module with one entry in this table.

*/

#include <libc.h>
#include "codegen.h"
#include "omf.h"

enum name_e {
	_nilname = 1,
	_data,
	xib,
	xi,
	xie,
	dgroup,
	data
};

static
char *segnames[] = {
	"",
	"_DATA",
	"XIB",
	"XI",
	"XIE",
	"DGROUP", 
	"DATA",
	0, 
};
	
static
_segdecl(FILE *fp, int name, int class, int len)
{
	orec_t	*r = omf_allocrec();
	omf_putbyte(r,(ALIGN_BYTE<<5)|(COMB_ADDOFF<<2)|1);
	omf_puteither(r, len);	/* len  */
	omf_putbyte(r, name);	/* name */
	omf_putbyte(r, class);	/* class (data) */
	omf_putbyte(r, 1);	/* overlay */
	omf_putrec(fp, r, CMD_SEGD32);
	omf_freerec(r);
}



mkxib(FILE *fp, char *modname, char *sym, char *pubnam, int prio, int needflt)
{
	orec_t	*r = omf_allocrec();

	omf_theader(fp, modname);
	omf_msobj(fp);

	if (prio == -1) {
		prio = 32;
	}

	omf_lnames(fp,segnames, 7);

	omf_putbyte(r,dgroup);	/* define dgroup */

	_segdecl(fp, _data, data, 0);
	omf_putbyte(r,GRP_SEGIDX); omf_putindex(r,1);

	_segdecl(fp, xib, data, 0);
	omf_putbyte(r,GRP_SEGIDX); omf_putindex(r,2);

	_segdecl(fp, xi, data, 6);
	omf_putbyte(r,GRP_SEGIDX); omf_putindex(r,3);

	_segdecl(fp, xie, data, 0);
	omf_putbyte(r,GRP_SEGIDX); omf_putindex(r,4);
/* group definition */
	omf_putrec(fp, r, CMD_GRPDEF);
	omf_clear(r);

	omf_putname(r, sym, strlen(sym));
	omf_putindex(r,0);
	if (needflt) { /* these names load floating point support */
		omf_putname(r,"__8087", strlen("__8087"));
		omf_putindex(r,0);
		omf_putname(r,"_fltused_", strlen("_fltused_"));
		omf_putindex(r, 0);
	}
	omf_putrec(fp,r,CMD_EXTDEF);
	omf_clear(r);

/* define the segment contents */
	omf_putindex(r, 3);	/* segment XI */
	omf_puteither(r, 0);	/* offset */

	omf_putbyte(r, 0);	/* first byte is reserved */
	omf_putbyte(r, prio);	/* priority */
	omf_putdword(r, 0);	/* offset of the item */
	omf_putrec(fp,r,CMD_LEDA32);

	omf_clear(r);

/* define fixup record for external symbol */

	omf_putbyte(r, FIXUPP_FIXUP | (LOC_MS_OFFSET_32 << 2) | FIXDAT_MBIT);
	omf_putbyte(r, 2);	/* offset in section  */
	omf_putbyte(r, (F_EXT<<4)|T_EXTWD|0x04);
	omf_putindex(r, 1);	/* Symbol number */
	omf_putindex(r, 1);	/* Symbol number */
	omf_putrec(fp, r, CMD_FIXU32);
	omf_clear(r);

/* put public def for this name */
	omf_putbyte(r,0);
	omf_putbyte(r, 3);	/* code segment */
	omf_putname(r, pubnam, strlen(pubnam));
	omf_puteither(r,0);
	omf_putindex(r,0);
	omf_putrec(fp,r,CMD_PUBD32);

	omf_freerec(r);

	omf_modend(fp, 0);
	return 0;
}

#if defined DEBUGGING
main()
{
	gen_xib(stdout,"--stdout--", "_slib_init_", "_dummyvar");
}
#endif


