/*-
 * simple macro handler
 */

#include <libc.h>
#include <sys/stat.h>
#include "skeleton.h"

#ifndef MAX_MAC
#define MAX_MAC 64
#endif

static struct {
	char           *name;
	char           *body;
}               mactab[MAX_MAC];
static int      nmac = 0;


char           *
getmac(char *name)
{
	int             i;
	for (i = 0; i < nmac && strcmp(mactab[i].name, name); i++);
	return i == nmac ? 0 : mactab[i].body;
}

static int
parse_skel(char *p)
{
	while (1) {
		char           *s;
		while (*p && strncmp(p, "@@", 2)) {
			if (p = strchr(p, '\n'))
				p++;
		}
		if (!p)
			break;
		mactab[nmac].name = s = p + 2;
		if (!(s = strchr(s, '{')))
			break;
		*s++ = '\0';
		mactab[nmac].body = s;
		if (!(s = strstr(s, "\n@@}\n")))
			break;
		*s = '\0';
		p = s + 4;
		nmac++;
	}
	return 0;
}

int
loadskel(char *fname)
{
	int             fd;
	struct stat     st;
	char           *p;
	int             n;
	if ((fd = open(fname, O_RDONLY)) == -1) {
		return -1;
	}
	if (fstat(fd, &st) == -1) {
		close(fd);
		return -1;
	}
	if ((p = calloc(1, st.st_size + 1)) == 0) {
		close(fd);
		return -1;
	}
	if (read(fd, p, st.st_size) != st.st_size) {
		close(fd);
		return -1;
	}
	n = parse_skel(p);
	close(fd);
	return n;
}


static
emits(char *bufp, int (*put) (int c, void *a), void *a)
{
	int             i;
	for (i = 0; bufp[i]; i++)
		(*put) (bufp[i], a);
	return i;
}

_macro(char *src, int nstr, char **strlist,
       int (*put) (int c, void *a), void *arg)
{
	int             i;
	int             r = 0;
	for (i = 0; i < nstr; i++) {
		char            buf[32];
		r += emits("#define __SHOBJ_ARG_", put, arg);
		ltoa(i, buf, 10);
		r += emits(buf, put, arg);
		(*put) (' ', arg);
		r++;
		r += emits(strlist[i], put, arg);
		(*put) ('\n', arg);
		r++;
	}
	while (*src) {
		(*put) (*src++, arg);
		r++;
	}
	return r;
}

static
putstr(int c, char **dest)
{
	*(*dest++) = c;
	**dest = 0;
	return c;
}

macro(char *src, char *dest, char **strlist)
{
	int             i;
	for (i = 0; strlist[i]; i++);
	return _macro(src, i, strlist, putstr, &dest);
}

//returns number of bytes written.
fmacro(FILE * dest, char *src, char **strlist)
{
	int             i;
	for (i = 0; strlist[i]; i++);
	return _macro(src, i, strlist, putc, dest);
}

fmacrov(FILE * fp, char *src,...)
{
	va_list         varg;
	char          **p;
	va_start(varg, src);
	p = (char **) varg[0];
	return fmacro(fp, src, p);
}




#ifdef UNIT_TEST
main()
{
	char            buf[80];
	char           *p;
	loadskel("./mklib.skel");

	while (gets(buf)) {
		p = getmac(buf);
		printf("%s = %s\n", buf, p);
	}
}
#endif
