#include <libc.h>
#include "mkshlib.h"
#include "map.h"

static maptab  *symtable;

static
init_symtab()
{
	if (!symtable)
		symtable = map_new(5001);
}

Symbol         *
sym_lookup(char *name)
{
	Symbol         *p;
	init_symtab();
	p = map_lookup(symtable, name);
	return p;
}

Symbol         *
sym_install(char *name, Symbol * p)
{
	init_symtab();
	return map_install(symtable, name, p);
}

char           *
sym_cname(char *name)
{
	static char     buf[256];
	extern char    *ass_type;
	Symbol         *p = sym_lookup(name);
	if (p) {
		if (p->is_code) {
			sprintf(buf, "%s()", name);
			return buf;
		}
		if (p->is_data) {
			strcpy(buf, name);
			return buf;
		}
	}
	sprintf(buf, "%s%s", name, ass_type);
	diag_msg(DIAG_INFOLEVEL, "symbol %s has default type (\'C\' symbol = %s)\n",
		name, buf);
	return buf;
}
char           *
sym_ename(char *name)
{
	static char     buf[256];
	extern char    *ass_type;
	Symbol         *p = sym_lookup(name);
	if (p) {
		if (p->is_code || p->is_branch) {
			sprintf(buf, "%s_", name);
			return buf;
		}
		if (p->is_data) {
			sprintf(buf, "_%s", name);
			return buf;
		}
	}
	sprintf(buf, *ass_type ? "%s_" : "_%s", name);
	diag_msg(DIAG_INFOLEVEL, "symbol %s has default type (linker symbol = %s)\n",
		name, buf);
	return buf;
}

Symbol         *
new_symbol()
{
	return (Symbol *) xcalloc(1, sizeof(Symbol), "new_symbol");
}

int
sym_apply(int (*f) (), void *p)
{
	return map_apply(symtable, f, p);
}

int
code_symbol(char *name)
{
	Symbol         *p = sym_lookup(name);
	if (!p) {
		p = new_symbol();
		sym_install(name, p);
	} else if (p->is_data) {
		warning("warning '%s' defined as data, redefined as code ", name);
	}
	p->is_data = 0;
	p->is_code = 1;
	return 0;
}

int
data_symbol(char *name)
{
	Symbol         *p = sym_lookup(name);
	if (!p) {
		p = new_symbol();
		sym_install(name, p);
	} else if (p->is_data) {
		warning("'%s' defined as code, redefined as data", name);
	}
	p->is_data = 1;
	p->is_code = 0;
	return 0;
}

int
anon_symbol(char *name)
{
	Symbol         *p = sym_lookup(name);
	if (!p) {
		p = new_symbol();
		sym_install(name, p);
	}
	return 0;
}

int
sect_symbol(char *name, long addr)
{
	Symbol         *p = sym_lookup(name);
	if (!p) {
		p = new_symbol();
		sym_install(name, p);
	} else {
		warning("redefining symbol '%s' to be section @ 0x%lx\n",
		     name, addr);
	}
	p->is_sect = 1;
	p->address = addr;
	return 0;
}

int
export_symbol(char *name)
{
	Symbol         *p;
	if (!*name)
		return 0;
	if (!(p = sym_lookup(name))) {
		p = new_symbol();
		p->is_data = 1;
		sym_install(name, p);
	}
	p->is_exported = 1;
	return 0;
}

